/* SAMPLE EYELINK EXPT FOR WIN32 PLATFORM  */
/*      (c) 1997 by SR Research Ltd.       */
/*      27 November '97 by Dave Stampe     */
/*       For non-commercial use only       */
/*                                         */
/*   This file can serve as a start        */
/*   for programming most experiments      */

/* Windows 95/NT demonstration experiment (C version) */
/* This file: Real-time gaze cursor uses data from link */
/* Uses most recently received sample only:             */
/* see playback_trial for how to get all data from link */

/* This module is intended only for private development of experiments       */
/* Use is granted for non-commercial applications by Eyelink licencees only  */

#include <windows.h>
#include <windowsx.h>
#include <stdio.h>
#include <math.h>

#include "eyelink.h"
#include "w32_mp_exptsppt.h"
#include "w32_mp_demo.h"  /* header file for this experiment */
#include "ellip.h"
#include "obj_gcwindow.h"

#define SHOW_DURATION   1000
#define BLANK_DURATION   100

/********* PERFORM AN EXPERIMENTAL TRIAL  *******/


// End recording: adds 100 msec of data to catch final events
static void end_trial(int speed)
{
  //(*dura_entries) -= 1;  
  while(getkey()) {};
}

/* Run a single trial, recording to EDF file and sending data through link */
/* This example draws to a bitmap, then copies it to display for fast stimulus onset */ 

// The order of operations is:
// - Set trial title, ID for analysis
// - Draw foreground, background bitmaps and create EyeLink display graphics
// - Drift correction
// - start recording
// - <DON'T copy bitmap to display: draw window when first sample arrives>
// - loop till button press, timeout, or abort, drawing gaze contingent window
// - stop recording, dispose of bitmaps, handle abort and exit

// NEW CODE FOR GAZE CONTINGENT DISPLAY:
// - create both foreground and background bitmaps
// - uses the eyelink_newest_float_sample() to get latest update
// - initialize window with initialize_gc_window()
// - moves window with redraw_gc_window()

// Run gaze-contingent window trial
// <fgbm> is bitmap to display within window
// <bgbm> is bitmap to display outside window
// <wwidth, wheight> is size of window in pixels
// <mask> flags whether to treat window as a mask
// <time_limit> is the maximum time the stimuli are displayed 

  UINT32 trial_start;	// trial start time (for timeout) 
  int button;		// the button pressed (0 if timeout) 
  int error;            // trial result code
  int key;              // key pressed
  
  ALLF_DATA evt, fix_evt; // buffer to hold sample and event data
  int eye_used = -1;      // indicates which eye's data to display
  float x, y;		  // gaze position 
  
  long int i, image_x, image_y, obj_x, obj_y, obj_c, obj_f, obj_t, objects;
  char text[20], temptext[30], gc_eye_file[80];
  FILE *eye_stream;

  HDC hdc;
  HBRUSH hbrush;
  HBITMAP trace_bitmap;
  int trial_on, f_start, f_end, f_half, f_half_starttime, f_half_endtime, f_lasthalf = -1, data_counter = 0;
  double fx, fy;
  double target_y, target_trick = 0;
	float eye_x, eye_y, pupil;
  int speed = 0, target_reached = 0, enter_timecode = -1, enter_timecode2 = -1, timecode, 
		  extra_param;
  int window_state, next_statechange, gc_display_type, gc_eye_display, eye_timer, next_eyetime;

  // NOTE: TRIALID AND TITLE MUST HAVE BEEN SET BEFORE DRIFT CORRECTION!
  // FAILURE TO INCLUDE THESE MAY CAUSE INCOMPATIBILITIES WITH ANALYSIS SOFTWARE!
  
  
  // Set size and type of gaze-contingent window 
  RECT display_rect;
  

void open_eye_file(void)
{
	eye_stream = fopen(gc_eye_file, "rt");
	fscanf(eye_stream, "%li %li", &eye_timer, &next_eyetime);
	eye_x = -500;
	eye_y = -500;
	eye_file_open = 1;
}


void close_eye_file(void) 
{
	fclose(eye_stream);
	eye_file_open = 0;
	eye_x = -500;
	eye_y = -500;
}


void init_display(HBITMAP fgbm1, HBITMAP fgbm2, HBITMAP bgbm, int display_type,
                 int wwidth, char *eye_file, int eye_display, int extra)
{
  display_rect.top    = dispinfo.top;
  display_rect.bottom = dispinfo.bottom;
  display_rect.left   = dispinfo.left;
  display_rect.right  = dispinfo.right;
	gc_display_type = display_type;
	gc_eye_display = eye_display;
	extra_param = extra;
	strcpy(gc_eye_file, eye_file);
	if ((display_type & 1) == 0)
	{
    initialize_ellip_gc_window(wwidth, wwidth, 0, 0, bgbm, bgbm, fgbm1, fgbm2,
      full_screen_window, display_rect, 0, 2, eye_display);
		trace_bitmap = fgbm1;
	}
	else
	{
    initialize_ellip_gc_window(wwidth, wwidth, 0, 0, fgbm1, fgbm2, bgbm, bgbm,
      full_screen_window, display_rect, 0, 2, eye_display);
    trace_bitmap = bgbm;
	}
  window_state = 0;
  next_statechange = SHOW_DURATION;
	trial_start = GetTickCount();  // record the display onset time 
	if (eye_display > 0)
	 open_eye_file();
}

int UpdateDisplay(int x, int y)
{
  HPEN hpen;
	HDC hdc, mdc;
  HBITMAP obm;
  
	int x_old = -999, y_old = -999;

	  if ((gc_display_type & 2) == 2 && GetTickCount() >= trial_start + next_statechange)
    {
      switch (window_state)
      {
      case 0: if (extra_param == 1)
							{
				        window_state = 2;
                next_statechange += SHOW_DURATION;
							}
				      else
							{
                window_state = 1;
                next_statechange += BLANK_DURATION;
							}
              break;
      case 1: window_state = 2;
              next_statechange += SHOW_DURATION;
              break;
      case 2: if (extra_param == 1)
							{
				        window_state = 0;
                next_statechange += SHOW_DURATION;
							}
				      else
							{
                window_state = 3;
                next_statechange += BLANK_DURATION;
							}
              break;
      case 3: window_state = 0;
              next_statechange += SHOW_DURATION;
      }
    }
    if (gc_eye_display > 0)
		{
			if (gc_eye_display == 3)
			{
		    hdc = GetDC(NULL);               
        mdc = CreateCompatibleDC(hdc);
        obm = SelectObject(mdc, trace_bitmap);     
        hpen = CreatePen(PS_SOLID, 4, RGB(255, 100, 255));
		    SelectObject(mdc, hpen);
		    while (next_eyetime != -1)
				{
				  fscanf(eye_stream, "%f %f %f %li", &eye_x, &eye_y, &pupil, &next_eyetime);
          if (extra_param == 2)
					{
					  x = (int) (eye_x*1.25 + 0.5);
			      y = (int) (eye_y*1.25 + 0.5);
					}
					else if (extra_param == 3)
					{
					  x = (int) (eye_x*1.25 + 25.5);
			      y = (int) (eye_y*1.25 + 0.5);
					}
					else 
					{
					  x = (int) (eye_x + 0.5);
			      y = (int) (eye_y + 0.5);
					}
					if (x_old > -900)				
					{
						MoveToEx(mdc, x_old, y_old, NULL);
			      LineTo(mdc, x, y);
					}
					x_old = x;
					y_old = y;
				}
				ReleaseDC(NULL, hdc);
        SelectBitmap(mdc, obm);      // Release GDI resources for BITMAP context
        DeleteDC(mdc);  
		    close_eye_file();
			  gc_eye_display = 0;
			}
			else
			{
		  while (next_eyetime != -1 && next_eyetime - eye_timer < GetTickCount() - trial_start)
				fscanf(eye_stream, "%f %f %f %li", &eye_x, &eye_y, &pupil, &next_eyetime);
        if (next_eyetime == -1)
				{
				  close_eye_file();
				  if (gc_eye_display == 2)
					{
					  open_eye_file();
					  trial_start = GetTickCount();  // record the display onset time 
					}
				}
        if (extra_param == 2)
				{
				  x = (int) (eye_x*1.25 + 0.5);
		      y = (int) (eye_y*1.25 + 0.5);
				}
				else if (extra_param == 3)
				{
				  x = (int) (eye_x*1.25 + 25.5);
		      y = (int) (eye_y*1.25 + 0.5);
				}
				else
				{
				  x = (int) (eye_x + 0.5);
		      y = (int) (eye_y + 0.5);
				}
			}
		}
		if (gc_eye_display != -1)
      redraw_ellip_gc_window(x, y, window_state, gc_eye_display);  // move window if visible      
	
  // report response result: 0=timeout, else button number
  // Call this at the end of the trial, to handle special conditions
  return 0;
}

