/* SAMPLE EYELINK EXPT FOR WIN32 PLATFORM  */
/*      (c) 1997 by SR Research Ltd.       */
/*      27 November '97 by Dave Stampe     */
/*                                         */
/*   Code for elliptical gaze-contingent   */
/*          windows, Version 1.1           */
/*      28 August '98 by Marc Pomplun      */
/*                                         */
/*       For non-commercial use only       */
/*                                         */
/*     This file can serve as a start      */
/*    for programming most experiments     */

/* Windows 95/NT demonstration experiment (C version) */
/* Draws gaze-contingent window, requires foreground and background bitmaps */

/* THIS CODE IS THE PROPERTY OF SR Research Ltd, Toronto, */
/* AND MAY NOT BE USED FOR COMMERCIAL PURPOSES WITHOUT PERMISSION */

/* This module is intended only for private development of experiments       */
/* Use is granted for non-commercial applications by Eyelink licencees only  */


#include <windows.h>
#include <windowsx.h>
#include <math.h>

#include "eyelink.h"

#define  SQR(x)   ((x)*(x))
#define  MIN(a,b) ((a<b)?(a):(b))
#define  MAX(a,b) ((a>b)?(a):(b))

#define  MAXHEIGHT    1000     // maximum height of ellipses to be drawn

#define  COPY_LINE_INIT -1     // parameters for the copy_line function
#define  COPY_LINE_DONE -2

#define  INITIALIZE      1      // window states
#define  DRAWN           2
#define  NOT_DRAWN       3
#define  FULL_SCREEN     4


/********** FAST ELLIPTICAL GAZE-CONTINGENT WINDOW ************/

// This gaze-contingent window uses differential updates for speed.
// If the window moves a small amount, then only a small area
// of the display will actually need to be redrawn.
// These functions identify and update the regions 
// that need to be redrawn using the foreground and background bitmaps.

static int window_w = 90;     // the width of the window in pixels
static int window_h = 75;     // The height of the window in pixels
static int window_xoffset;    // offset between window and gaze position in x-
static int window_yoffset;    // and in y-direction
static int window_mask;       // window might be a mask

static int curr_wl, curr_wr;  // the sides of the window as currently drawn
static int curr_wt, curr_wb;

static int wl, wr, wt, wb;    // the coordinates of the window to be drawn

static int window_state;      // current state of the gc window

static int last_x, last_y;    // center of current window
static int wdeadband;         // size of anti-jitter deadband

static int ellipse_left[MAXHEIGHT], ellipse_right[MAXHEIGHT], 
ellipse_width[MAXHEIGHT];
// arrays containing ellipse shape
// for fast drawing
static int area_left[2*MAXHEIGHT], area_top[2*MAXHEIGHT], 
area_width[2*MAXHEIGHT], area_height[2*MAXHEIGHT], areas;
// arrays containing areas to be drawn or erased
static HWND ell_hwnd;
static RECT dr;            // display rectangle
static HBITMAP ell_fgbm, ell_fgbm1, ell_fgbm2;   // foreground bitmap
static HBITMAP ell_bgbm, ell_bgbm1, ell_bgbm2;   // background bitmap
static HDC ell_hdc;        // static global DC variables for speed optimization
static HDC mdc_fg, mdc_bg;
static HBITMAP obm_fg, obm_bg;
static HPEN hpen;
static int last_display_state, line_drawn;


static void display_rectangular_bitmap(HBITMAP hbm, int xs1, int ys1, int xs2, int ys2)
{
  HDC hdc;
  HDC mdc;
  HBITMAP obm;
  int h, w;
  
  if(!ell_hwnd || !hbm) return;
  w = xs2 - xs1 + 1;
  h = ys2 - ys1 + 1; 
  
  hdc = GetDC(ell_hwnd);            // display drawing context
  mdc = CreateCompatibleDC(hdc);    // memory context
  obm = SelectObject(mdc, hbm);     // select bitmap
  
  BitBlt(hdc, xs1, ys1, w, h, mdc, xs1, ys1, SRCCOPY);   // copy relevant part to display
  
  SelectBitmap(mdc, obm);           // release GDI resources
  DeleteDC(mdc);
  ReleaseDC(ell_hwnd, hdc);
}



// Initial setup of gaze-contingent window before drawing it.
// Sets size of window and offset of its center from gaze position
// <deadband> sets number of pixels of anti-jitter applied  

void initialize_ellip_gc_window(int wwidth, int wheight, int xoffset, int yoffset,
                                HBITMAP window_bitmap1, HBITMAP window_bitmap2, 
                                HBITMAP background_bitmap1, HBITMAP background_bitmap2,
                                HWND window, RECT display_rect, int is_mask, int deadband,
																int eye_display)    
{
  int line;
  double center_x, center_y, size_x;
  
  window_w = wwidth;          // set window parameters
  window_h = wheight;
  window_xoffset = xoffset;
  window_yoffset = yoffset;
  window_mask = is_mask;      // Window or mask? In case of missing data, a mask will
  // cover the whole screen, whereas a window will disappear.
  // When a mask is moved, the painting is done before the
  // erasing, while a window is first erased and then painted. 
  // This way, the subject cannot perceive more information
  // during eye movements than specified by the window size.
  
  window_state = INITIALIZE;  // draw background image
  last_display_state = -1;
  dr = display_rect;
  ell_hwnd = window;
  ell_fgbm1 = window_bitmap1;
  ell_bgbm1 = background_bitmap1;
  ell_fgbm2 = window_bitmap2;
  ell_bgbm2 = background_bitmap2;
  line_drawn = 0;  
  wdeadband = deadband;
  last_x = last_y = MISSING;
  
  center_x = (double) wwidth/2.0;          // calculate ellipse
  center_y = (double) wheight/2.0;               
  for (line = 0; line < wheight; line++)   // write table containing ellipse shape
  {                                        // for every horizontal line of ellipse
    size_x = center_x*sqrt(1 - SQR(fabs((double) line + 0.5 - center_y)/center_y));
    ellipse_left[line] = (int) (center_x - size_x + 0.5);    // x-position of left boundary 
    ellipse_right[line] = (int) (center_x + size_x - 0.5);   // x-position of right boundary
    ellipse_width[line] = ellipse_right[line] - ellipse_left[line] + 1;   // width
  }                                                    
	if (eye_display == 3)
		display_rectangular_bitmap(ell_bgbm, dr.left, dr.top, dr.right, dr.bottom);
}



/************* DISPLAY RECTANGULAR PART OF BITMAP **********/

// Copies a rectangular area of a bitmap to the display
// Copies rectangle <xs1, ys1, xs2, ys2> of DDB <hbm> 
// <hbm> is source bitmap, <hwnd> is the window to draw to
// This function is used for screen initializing and full ellipse erasing



/************** MOVE ELLIPTICAL WINDOW ON SCREEN **************/

// Receives rectangular area that is to be displayed as window area.
// <draw_x, draw_y> ist upper left corner, width and height are provided
// as well. Due to the symmetry between areas to be drawn and erased for
// window movement, draw_or_erase can calculate the corresponding
// area to be erased (displayed as background) and save this information
// in arrays. In a second step, copy_line will process these areas.
// If window is a mask, the new window area is painted first and the old
// window area erased afterwards. If not, the order is reversed.

static void draw_or_erase(int draw_x, int draw_y, int width, int height) 
{	  	
  static int erase_x, erase_y;
  
  erase_x = wl + curr_wr - width + 1 - draw_x;   // calculate top-left corner of
  erase_y = wt + curr_wb - height + 1 - draw_y;  // opposite rectangle for second step
  if (window_mask)                               // if mask, draw first...
  {
    BitBlt(ell_hdc, draw_x, draw_y, width, height, mdc_fg, draw_x, draw_y, SRCCOPY);
    area_left[areas] = erase_x;                  // ... and remember the areas to be
    area_top[areas] = erase_y;                   //     erased in the second step.
  }
  else                                           // if normal window, erase first...
  {
    BitBlt(ell_hdc, erase_x, erase_y, width, height, mdc_bg, erase_x, erase_y, SRCCOPY);
    area_left[areas] = draw_x;                   // ... and remember the areas to be
    area_top[areas] = draw_y;                    //     drawn in the second step.
  }
  area_width[areas] = width;                     // remember width and height
  area_height[areas] = height;
  areas++;
}


// For speed optimization, the horizontal lines of areas to be updated
// should - if possible - be combined to rectangles in order to reduce the
// number of BitBlt calls. Copy_line receives horizontal lines to be
// updated (starting at <x, y>, having <width> pixels) and tries to combine 
// them to rectangles. Both rectangles and incompatible lines are sent to 
// draw_and_erase. For each window movement, copy_line must be initialized with 
// <width> = COPY_LINE_INIT and receive a final call with
// <width> = COPY_LINE_DONE. This final call executes the second step of
// window updating (erasing or drawing, depending on window_mask). 

static void copy_line(int x, int y, int width) 
{
  static int area1_top, area1_bottom, area1_left, area1_width,
    area2_top, area2_bottom, area2_left, area2_width, line_added;
  // there can only be two rectangles at a time that might be
  // extended by further lines sent to copy_line
  static int i;
  HDC mdc;
  
  if (width == COPY_LINE_INIT)          // initialize
  {
    area1_width = 0;
    area2_width = 0;
    areas = 0;
    return;
  }
  else if (width == COPY_LINE_DONE)     // finish: send remaining rectangles
  {                                     // to draw_and_erase
    if (area1_width > 0)                // is information left in area1?
      draw_or_erase(area1_left, area1_top, area1_width, area1_bottom - area1_top + 1);
    if (area2_width > 0)                // is information left in area2?
      draw_or_erase(area2_left, area2_top, area2_width, area2_bottom - area2_top + 1);
    
    if (window_mask)                    // Second step: if window is a mask,
      mdc = mdc_bg;                     // the action to be performed is erasing
    else
      mdc = mdc_fg;                     // otherwise, it is drawing
    for (i = 0; i < areas; i++)         // erase or draw the recorded areas, resp.
      BitBlt(ell_hdc, area_left[i], area_top[i], area_width[i], area_height[i], mdc, 
      area_left[i], area_top[i], SRCCOPY);    
    return;
  }
  line_added = 0;
  if (area1_width > 0 && y == area1_bottom + 1 &&        // can the line be appended to
    x == area1_left && width == area1_width)           // area1?
  {
    area1_bottom++;                                      // -> increase area1 
    line_added = 1;                                      // remember: line added
  }
  else if (area2_width > 0 && y == area2_bottom + 1 &&   // can the line be appended to
    x == area2_left && width == area2_width)      // area2?
  {
    area2_bottom++;                                      // -> increase area2
    line_added = 1;                                      // remember: line added
  }
  if (line_added == 0)                                 
  {                                
    // if line could not be appended, start new area. Use area1, if area1 is empty or
    // if the bottom of area1 is above the bottom of area2 or if area1 is to the
    // left of area2
    if (area1_width == 0 || (area1_width > 0 && area2_width > 0 && 
      1000*area1_bottom + area1_left < 1000*area2_bottom + area2_left)) 
    {
      if (area1_width > 0) // if area1 is not empty, send it to draw_and_erase
        draw_or_erase(area1_left, area1_top, area1_width, area1_bottom - area1_top + 1);
      area1_top = y;       // start new area1
      area1_bottom = y;
      area1_left = x;
      area1_width = width;
    }
    else     // use area2
    {
      if (area2_width > 0)  // if area2 is not empty, send it to draw_and_erase
        draw_or_erase(area2_left, area2_top, area2_width, area2_bottom - area2_top + 1);
      area2_top = y;        // start new area2
      area2_bottom = y;
      area2_left = x;
      area2_width = width;
    }
  }
}


// Copies an elliptical area of a bitmap to the display.
// Copies ellipse with left and top coordinates <xs1, ys1>
// This function is used if the whole window has to be painted,
// i.e., the window was not previously shown

static void display_elliptical_bitmap(int xs1, int ys1)
{
  HDC hdc;
  HDC mdc;
  HBITMAP obm;
  static int line;
  
  hdc = GetDC(ell_hwnd);                  // display drawing context
  mdc = CreateCompatibleDC(hdc);          // memory context
  obm = SelectObject(mdc, ell_fgbm);      // select bitmap
  
  for (line = 0; line < window_h; line++) // draw line by line using pre-calculated arrays
    BitBlt(hdc, xs1 + ellipse_left[line], ys1 + line, ellipse_width[line], 1, 
    mdc, xs1 + ellipse_left[line], ys1 + line, SRCCOPY);   
  
  SelectBitmap(mdc, obm);                 // release GDI resources
  DeleteDC(mdc);
  ReleaseDC(ell_hwnd, hdc);
}


// updates the display according to the (global) new window
// coordinates <wl, wt, wr, wb> and the previous coordinates
// <curr_wl, curr_wt, curr_wr, curr_wb>.

static void move_ellip_gc_window(void)
{
  int line, curr_line, bl, br, curr_bl, curr_br;
  
  ell_hdc = GetDC(ell_hwnd);                 // display drawing context
  mdc_fg = CreateCompatibleDC(ell_hdc);      // memory contexts for foreground (window)
  mdc_bg = CreateCompatibleDC(ell_hdc);      // and background
  obm_fg = SelectObject(mdc_fg, ell_fgbm);   // select bitmaps
  obm_bg = SelectObject(mdc_bg, ell_bgbm);
  copy_line(0, 0, COPY_LINE_INIT);           // initialize copy_line
  for (line = 0; line < window_h; line++)    // send area to be updated line by line
  {
    bl = wl + ellipse_left[line];      // left boundary (bl) and
    br = wl + ellipse_right[line];     // right boundary (br) of new window
    curr_line = line - curr_wt + wt;   // line corresponds to curr_line of old 
    // window position
    if (curr_line < 0 || curr_line >= window_h)     // if there is no overlap,
      copy_line(bl, wt + line, ellipse_width[line]);
    // draw the whole line
    else 
    {
      curr_bl = curr_wl + ellipse_left[curr_line];  // left and right boundaries
      curr_br = curr_wl + ellipse_right[curr_line]; // of old window
      if (curr_br < bl || curr_bl > br)             // if no overlap,
        copy_line(bl, wt + line, ellipse_width[line]);
      // draw the whole line
      else 
      { 
        if (bl < curr_bl)  // if there is a new part to be drawn on the
          // left side of the old window, draw it
          copy_line(bl, wt + line, curr_bl - bl);			  
        if (br > curr_br)  // do the same for the right side
          copy_line(curr_br + 1, wt + line, br - curr_br);
      }
    }
  }
  copy_line(0, 0, COPY_LINE_DONE);
  //SelectBitmap(mdc_fg, obm_fg);           // release GDI resources for foreground
  //SelectBitmap(mdc_bg, obm_bg);           // release GDI resources for background
  DeleteDC(mdc_fg);
  DeleteDC(mdc_bg);
  ReleaseDC(ell_hwnd, ell_hdc);
}


// Draw elliptical GC window at a new location
// The first time window is drawn, 
// the background outside the window will be filled in too.
// If X or Y is MISSING_DATA (defined in eyelink.h), window is hidden. 

void redraw_ellip_gc_window(int x, int y, int display_state, int eye_display)
{
	if (eye_display == 3)
		return;

	if (display_state != last_display_state)
  {
    last_display_state = display_state;
    window_state = INITIALIZE;
	  switch (display_state)
    {
    case 0: ell_fgbm = ell_fgbm1;
            ell_bgbm = ell_bgbm1;
            break;
    case 1: clear_full_screen_window(RGB(255, 255, 255));
            break;
    case 2: ell_fgbm = ell_fgbm2;
            ell_bgbm = ell_bgbm2;
            break;
    case 3: clear_full_screen_window(RGB(255, 255, 255));
    }
  }
	
  if (display_state == 1 || display_state == 3)
    return;
  if (last_x == MISSING_DATA || x == MISSING_DATA) 
    last_x = x;
  else if (x < last_x - wdeadband)    // if x-movement exceeds deadband, update x
    x = last_x = x + wdeadband;
  else if (x > last_x + wdeadband) 
    x = last_x = x - wdeadband;
  else 
    x = last_x;                       // if not, do not change x
  
  if (last_y == MISSING_DATA || y == MISSING_DATA) 
    last_y = y;
  else if (y < last_y - wdeadband)    // if y-movement exceeds deadband, update y
    y = last_y = y + wdeadband;
  else if (y > last_y + wdeadband) 
    y = last_y = y - wdeadband;       
  else 
    y = last_y;                       // if not, do not change y
  
  if (window_state == INITIALIZE)             // for initialization,    
  {                                           // just fill in all of window
    display_rectangular_bitmap(ell_bgbm, dr.left, dr.top, dr.right, dr.bottom);
    window_state = NOT_DRAWN;
  }
  if (x == MISSING_DATA || y == MISSING_DATA) // if no data available,
  {
    if (window_mask && (window_state != FULL_SCREEN)) 
    {
      display_rectangular_bitmap(ell_fgbm, dr.left, dr.top, dr.right, dr.bottom);
      window_state = FULL_SCREEN;    // if window is a mask, hide all information
    }
    else if (!window_mask && (window_state != NOT_DRAWN))
    {
      display_rectangular_bitmap(ell_bgbm, curr_wl, curr_wt, curr_wr, curr_wb);
      window_state = NOT_DRAWN;      // otherwise, hide window
    }
  }
  else
  {
    wt = y + window_yoffset - window_h/2;          // compute new window edges
    wb = wt + window_h - 1;                        // use window offsets  
    wl = x + window_xoffset - window_w/2;
    wr = wl + window_w - 1;
    if (window_state == FULL_SCREEN)               // in case of full screen mask,
    {                                              // draw background and window
      display_rectangular_bitmap(ell_bgbm, dr.left, dr.top, dr.right, dr.bottom);
      display_elliptical_bitmap(wl, wt);
    }
    else if (window_state == NOT_DRAWN)            // if no window was shown,
      display_elliptical_bitmap(wl, wt);           // draw new window
    else                                           // otherwise,
      move_ellip_gc_window();                      // just move window
    
    curr_wl = wl;                                  // record new window edges.
    curr_wr = wr;
    curr_wt = wt;
    curr_wb = wb;
    window_state = DRAWN;
  }
}


