/*   EYELINK WINDOWS 95/NT EXPT SUPPORT      */
/*   (c) 1997 by SR Research                 */
/*   24 Nov '97 by Dave Stampe               */
/*   For non-commercial use only             */
/*				             */
/* Header file for EXPTSPPT.DLL              */

/* This module is for user applications   */
/* Use is granted for non-commercial      */
/* applications by Eyelink licencees only */

/************** WARNING **************/
/*                                   */
/* UNDER NO CIRCUMSTANCES SHOULD     */
/* PARTS OF THESE FILES BE COPIED OR */
/* COMBINED.  This will make your    */
/* code impossible to upgrade to new */
/* releases in the future, and       */
/* SR Research will not give tech    */
/* support for reorganized code.     */
/*                                   */
/* This file should not be modified. */
/* If you must modify it, copy the   */
/* entire file with a new name, and  */
/* change the the new file.          */
/*                                   */
/************** WARNING **************/


#ifdef __cplusplus 	/* For C++ compilers */
extern "C" {
#endif

#ifndef SIMLINKINCL
  #include "eyelink.h"
#endif

#ifndef W32EXPTSPPTINCL
#define W32EXPTSPPTINCL

/************ SYSTEM SETUP *************/

                // Set up the EyeLink system and connect to tracker
                // If <dummy> not zero, will create a dummy connection
                // where eyelink_is_connected() will return -1
                // Returns: 0 if success, else error code
INT16 open_eyelink_connection(INT16 dummy);

                // Close any EyeLink connection, release EyeLink system
void close_eyelink_connection(void);

                // Changes the multitasking proirity of current application
                // Using THREAD_PRIORITY_ABOVE_NORMAL may reduce latency
                // Reset priority with THREAD_PRIORITY_NORMAL
                // Too high priority will stop the link from functioning!
INT32 set_application_priority(INT32 priority);


/*********** KEYBOARD SUPPORT **********/

	/* some useful keys returned by getkey()                    */
	/* These keys allow remote control of tracker during setup. */
        /* on non-DOS platforms, you should produce these codes and */
	/* all printable (0x20..0x7F) keys codes as well.           */
	/* Return JUNK_KEY (not 0) if untranslatable key pressed.   */
        /* TERMINATE_KEY can be to break out of EXPTSPPT loops.     */

#define CURS_UP    0x4800
#define CURS_DOWN  0x5000
#define CURS_LEFT  0x4B00
#define CURS_RIGHT 0x4D00

#define ESC_KEY   0x001B
#define ENTER_KEY 0x000D

#define PAGE_UP   0x4900
#define PAGE_DOWN 0x5100

#define JUNK_KEY      1       /* returns this code if untranslatable key */
#define TERMINATE_KEY 0x7FFF  /* returns this code if program should exit */

	// EYELINK tracker (MS-DOS) key scan equivalent
        // Processes Windows messages, records key events
        /* Returns 0 if no key pressed */
        /* returns 1-255 for non-extended keys */
        /* returns 0x##00 for extended keys (##=hex code) */
UINT16 getkey(void);

         // Similar to getkey(), bt also sends key to tracker
         // Supports reporting of repeating keys
UINT16 echo_key(void);

         // Translate WM_KEYDOWN or WM_CHAR message into EyeLink key                                          
         // Returns 0 if not a translatable key, else EyeLink key code
UINT16 translate_key_message(UINT message, WPARAM wParam, LPARAM lParam);

          // Call with WM_CHAR and WM_KEYDOWN messages from window callback
          // Process messages, translates key messages and enqueues
UINT16 process_key_messages(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
    
          // Initializes and empties local key queue 
void flush_getkey_queue(void);

	  // Similar to getkey(), but doesnt call message pump
          // Use to build message pump for your own window
UINT16 read_getkey_queue(void);

/************* BREAK TESTS *********/

                // returns non-zero if ESC key held down
INT16 escape_pressed(void);

                // returns non-zero if Ctrl-C held down
INT16 break_pressed(void);


/********* ASYNCHRONOUS BREAKOUTS *********/

    // Because Windows is multi-tasking, some other event (i.e. a timer event or ALT-TAB)
    // may affect your application during loops or calibration.
    // Your event handlers can call these functions to stop ongoing operations

            // call from Windows event handlers when application must exit
            // forces calibration or drift correction to exit with result=27
            // when <assert> is nonzero,  will caused break_pressed() to test trua continuously, 
            // also causes getkey() to return TERMINATE_KEY
            // If <assert> is 0, will restore break_pressed() and getkey() to normal
void terminal_break(INT16 assert);

              // Call this to stop calibration/drift correction in progress
              // This could be called from a Windows message handler 
void exit_calibration(void);


/********* SYNC TO DISPLAY RETRACE ********/

            // wait for VGA card retrace to begin		
            // returns immediately if running under Windows NT
void wait_for_video_refresh(void);


/********** VIDEO DISPLAY INFORMATION **********/

        // This structure holds information on the display
        // Call get_display_infomation() to fill this with data
        // Check mode before running experiment!

#ifndef DISPLAYINFODEF
  #define DISPLAYINFODEF
  typedef struct {
           INT32 left;      // left of display
           INT32 top;       // top of display
           INT32 right;     // right of display
           INT32 bottom;    // bottom of display
           INT32 width;     // width of display
           INT32 height;    // height of display
           INT32 bits;      // bits per pixel
           INT32 palsize;   // total entries in palette (0 if not indexed display mode)
           INT32 palrsvd;   // number of static entries in palette (0 if not indexed display mode)
           INT32 pages;     // pages supported
           float refresh;   // refresh rate in Hz
           INT32 winnt;     // 0 if Windows 95, 1 if Windows NT
              } DISPLAYINFO;
#endif

      // you should put a global copy of this in your code 
extern DISPLAYINFO dispinfo;

#define SCREEN_LEFT   dispinfo.left
#define SCREEN_TOP    dispinfo.top
#define SCREEN_RIGHT  dispinfo.right
#define SCREEN_BOTTOM dispinfo.bottom
#define SCRHEIGHT     dispinfo.height
#define SCRWIDTH      dispinfo.width

          // get information on video driver and current mode
          // use this to determine if in proper mode for experiment.
          // If <di> not NULL, copies data into it
void get_display_information(DISPLAYINFO *di);


/************* PAGE SPPT ************/

      // Initialze graphics and calibration system
      // Handle to a full-screen window that will be used during experiment must be supplied
      // If you can't supply a handle, use NULL and make your window the topmost before call
      // If not NULL, the <info> struct wull be filled with information on the display mod
      // Returns 0 if OK
INT16 init_expt_graphics(HWND hwnd, DISPLAYINFO *info);

      // Close the calibration and graphics, release resources
      // You should only destroy your full-screen window after this call
void close_expt_graphics(void);

      // Gets a display context to draw on the page
      // This is a static DC, so all settings from your last use are preserved 
HDC get_window_dc(void);

      // Release the page DC 
INT16 release_window_dc(HDC hdc);

        // Clear a window to a defined color
        // Uses closest of current system palette colors
void clear_window(HWND hwnd, COLORREF color);


/********* CALIBRATION COLORS, TARGET, AND SOUNDS *******/

            // Sets target color and display background color
void set_calibration_colors(COLORREF fg, COLORREF bg);

            // Sets diameter of target and of hole in middle of target
            // Dimensions are in pixels
void set_target_size(UINT16 diameter, UINT16 holesize);

            // Selects sounds to be played on calibration events
            // If a sound is "", a default ound will be made
            // If a sound is "off", no sound is played
            // Otherwise, this is the name of a .WAV file.
            // If no sound card is present, system beeps will be substituted

            // Set sounds for Setup menu (calibration, validation)
            // New target display, successful operation, error or cancellation                                       
void set_cal_sounds(char *target, char *good, char *error);

            // Set sounds for drift correction
            // New target display, success, or ESC pressed (Setup begun)                                       
void set_dcorr_sounds(char *target, char *good, char *setup);

/********* ABORT CALIBRATION / DC LOOP ***********/

              // Call this to stop calibration/drift correction in progress
              // This could be called from a Windows message handler 
void exit_calibration(void);


/******** LINK FORMATTED COMMANDS **********/

	/* link command formatting */
	/* use just like printf() */
	/* returns command result */
	/* allows 500 msec. for command to finish */

INT16 eyecmd_printf(char *fmt, ...);

	/* link message formatting */
	/* use just like printf() */
	/* returns any send error */

INT16 eyemsg_printf(char *fmt, ...);


/*********** RECORDING SUPPORT FUNCTIONS ************/

      /* RETURN THESE CODES FROM YOUR RECORDING FUNCTIONS */
      /* These codes are returned by all these functions  */

#define DONE_TRIAL   0  /* return codes for trial result */
#define TRIAL_OK     0
#define REPEAT_TRIAL 1
#define SKIP_TRIAL   2
#define ABORT_EXPT   3

#define TRIAL_ERROR  -1 /* Bad trial: no data, etc. */

	       /* Start recording with data types requested      */
	       /* Check that all requested link data is arriving */
	       /* return 0 if OK, else trial exit code           */
INT16 start_recording(INT16 file_samples, INT16 file_events,
		   			  INT16 link_samples, INT16 link_events);

	/* Check if we are recording: if not, report an error  */
	/* Also calls record_abort_hide() if recording aborted */
        /* Returns 0 if recording in progress                  */
	/* Returns ABORT_EXPT if link disconnected             */
	/* Handles recors abort menu if trial interrupted      */
	/* Returns TRIAL_ERROR if other non-recording state    */

	/* Typical use is  */
        /*   if((error=check_recording())!=0) return error; */

INT16 check_recording(void);

       /* halt recording, return when tracker finished mode switch */
void stop_recording(void);

       /* enter tracker idle mode, wait  till finished mode switch */
void set_offline_mode(void);

       /* call at end of trial, return result                   */
       /* check if we are in Abort menu after recording stopped */
       /* returns trial exit code                               */
INT16 check_record_exit(void);


/********* PERFORM SETUP ON TRACKER  *******/

	/* Starts tracker into Setup Menu. */
	/* From this the operator can do camera setup, calibrations, etc. */
	/* Pressing ESC on the tracker exits. */
	/* Leaving the setup menu on the tracker (ESC) key) also exits. */
	/* RETURNS: 0 if OK, 27 if aborted, TERMINATE_KEY if pressed */
INT16 do_tracker_setup(void);


/********* PERFORM DRIFT CORRECTION ON TRACKER  *******/

   /* Performs a drift correction, with target at (x,y).     */
   /* If operator aborts with ESC, we assume there's a setup */
   /* problem and go to the setup menu (which may clear the  */
   /* display).  Redraw display if needed and repeat the     */
   /* call to  do_drift_correct() in this case.              */

   /* ARGS: x, y: position of target */
   /*       draw: draws target if 1, 0 if you draw target first */
   /*       allow_setup: 0 disables ESC key setup mode entry */
   /* RETURNS: 0 if OK, 27 if Setup was called, TERMINATE_KEY if pressed */

INT16 do_drift_correct(INT16 x, INT16 y, INT16 draw, INT16 allow_setup);


/*********** FILE TRANSFER **********/

  /* THIS ROUTINE MAY NEED TO BE CREATED FOR EACH PLATFORM */
  /* This call should be implemented for a standard programming interface */

   /* Copies tracker file <src> to local file <dest>.     */
   /* If specifying full file name, be sure to add ".edf" */
   /* extensions for data files.                          */

   /* If <src> = "", tracker will send last opened data file.            */
   /* If <dest> is NULL or "", creates local file with source file name. */
   /* Else, creates file using <dest> as name.  If <dest_is_path> != 0   */
   /* uses source file name but adds <dest> as directory path.
   /* returns: file size if OK, negative =  error code       */

INT32 receive_data_file(char *src, char *dest, INT16 dest_is_path);


/************* MESSAGE PUMP ************/
                                  
        // allows messages to operate in loops
	// returns nonzero if app terminated                          
	// eats key events, places on key queue for getkey()
        // getkey() and echo_key() also call message_pump()
              
       // calling this in loops allows Windows to process messages
       // returns nonzero if application terminated (ALT-F4 sent to window)
       // set <dialog_hook> to handle of modeless dialog 
       // in order to properly handle its messages as well
INT16 message_pump(HWND dialog_hook);

        // similar to message_pump(), but only processes keypresses
        // this may help reduce latency
INT16 key_message_pump(void);

        // Similar to msec_delay(), but allows Widows message process
        // only allows message processing if delay > 20 msec
        // does not process dialog box messages
void pump_delay(UINT32 del);


/************* GENERAL PURPOSE NUMBER/TEXT DIALOG **************/
                        
                // Create single-line text-edit DB
                // with window <title>, text message <msg>
                // text buffer <txt> contains original text, will get result
                // <maxsize> sets max number of characters  
                // Returns: 0 if OK, 1 if cancelled, -1 if ALT-F4 pressed
INT16 edit_dialog(HWND hwnd, LPSTR title, LPSTR msg, LPSTR txt, INT16 maxsize);

/********** ALERT BOX ************/

	// displays general STOP-icon alert box
	// text is formatted via printf-like arguments
void alert_printf(char *fmt, ...);


/********* EDF OPEN, CLOSE ************/

// These functions were added as future revisions of EyeLink
// might require significant time to open and close EDF files

	// Opens EDF file on tracker hard disk
	// Returns 0 if success, else error code
INT16 open_data_file(char *name);

	// Closes EDF file on tracker hard disk
	// Returns 0 if success, else error code
INT16 close_data_file(void);


#endif //W32EXPTSPPTINCL

#ifdef __cplusplus 	/* For C++ compilation */
}
#endif

/*************** GAZE-CONTINGENT WINDOW ************/

        // Initial setup of gaze-contingent window before drawing it.
        // Sets size of window, and whether it is a foveal mask.
        // If height or width is -1, the window will be a bar covering the display
        // Supply a bitmap for window and background, <size of display_rect>
        // <window> is the window to display in
        // <display_rect> should be area of display in window.
        // <deadband> sets number of pixels of anti-jitter applied  
void initialize_gc_window(int wwidth, int wheight, 
                          HBITMAP window_bitmap, HBITMAP background_bitmap,
                          HWND window, RECT display_rect, int is_mask, int deadband);


       // Draw GC window at a new location
       // The first time window is drawn, 
       // the background outside the window will be filled in too.
       // If X or Y is MISSING_DATA (defined in eyelink.h), window is hidden. 
void redraw_gc_window(int x, int y);
    