/* SAMPLE EYELINK EXPT FOR WIN32 PLATFORM  */
/*      (c) 1997 by SR Research Ltd.       */
/*      27 November '97 by Dave Stampe     */
/*       For non-commercial use only       */
/*                                         */
/*   This file can serve as a start        */
/*   for programming most experiments      */

/* Windows 95/NT demonstration experiment (C version) */
/* This file: Read a .BMP file and create a bitmap */

/* This module is intended only for private development of experiments       */
/* Use is granted for non-commercial applications by Eyelink licencees only  */

#include <windows.h>
#include <windowsx.h>

#include "eyelink.h"
#include "w32_mp_exptsppt.h"
#include "w32_mp_demo.h"  /* header file for this experiment */


/*********** LOAD BMP FILE, CREATE BITMAP ************/

           // Creates full-screen sized DDB bitmap from loaded .BMP file
           // If <dx, dy> are not 0, will resize image to those dimensions
           // If image won't fill display, clear bitmap to <bgcolor> and centers it
HBITMAP bmp_file_bitmap(char *fname, int dx, int dy, COLORREF bgcolor)
{
  HDC hdc;
  HDC mdc, dmdc;
  HBITMAP hbm;
  HBITMAP hdibm;
  HBITMAP obm, odbm;
  DIBSECTION ds;
  int x=0, y=0;
              
                      // load the BMP file
  hdibm = (HBITMAP)LoadImage(NULL, fname, IMAGE_BITMAP, dx, dy, LR_LOADFROMFILE|LR_CREATEDIBSECTION);
  if(!hdibm) 
    {
      return NULL;
    }
  GetObject(hdibm, sizeof(DIBSECTION), &ds);   // Get information
  if(dx==0 && dy==0)                           // set default size if required
    {
      dx = ds.dsBm.bmWidth;
      if(dx > SCRWIDTH) dx = SCRWIDTH;
      dy = ds.dsBm.bmHeight;
      if(dy > SCRHEIGHT) dy = SCRHEIGHT;
 
  }
  x = (SCRWIDTH-dx) / 2;      // compute top-left to center bitmap if too small
  y = (SCRHEIGHT-dy) / 2;

  hdc = GetDC(NULL);               
  mdc = CreateCompatibleDC(hdc);    // Create a memory context for BITMAP
  dmdc = CreateCompatibleDC(hdc);   // Create a memory context for DIBSECTION

  hbm = CreateCompatibleBitmap(hdc, SCRWIDTH, SCRHEIGHT);  // Create the BITMAP
  if(!hbm) 
    {
      DeleteObject(hdibm);           // Clean up if failed
      DeleteDC(dmdc);
      DeleteDC(mdc);
      ReleaseDC(NULL, hdc);
      return NULL;
    }
  obm = SelectObject(mdc, hbm);        // select BITMAP
  odbm = SelectBitmap(dmdc, hdibm);    // select DIBSECTION

  if(x>0 || y>0)              // do we need to clear background?
    {
      HBRUSH oBrush = SelectObject(mdc, CreateSolidBrush(bgcolor | 0x02000000L));  // brush to fill with 
      PatBlt(mdc, 0, 0, SCRWIDTH, SCRHEIGHT, PATCOPY);
      DeleteObject(SelectObject(mdc, oBrush));
    }

  StretchBlt(mdc, x, y, dx, dy, dmdc,  // Copy, convert and size DIBSECTION image
             0, 0, ds.dsBm.bmWidth, ds.dsBm.bmHeight, SRCCOPY);

  SelectBitmap(dmdc, odbm);   // Release GDI resources for DIBSECITON context
  DeleteDC(dmdc);
  DeleteObject(hdibm);        // Delete loaded DIBSECTION

  SelectBitmap(mdc, obm);      // Release GDI resources for BITMAP context
  DeleteDC(mdc);
  ReleaseDC(full_screen_window, hdc);

  return hbm;
} 
 

void add_file_bitmap(HBITMAP bm_big, char *fname, int x, int y)
{
  HDC hdc_add, hdc_big;
  HDC mdc_add, mdc_big;
  HBITMAP bm_add;
  DIBSECTION ds;
  int dx, dy;

                      // load the BMP file to be added
  bm_add = (HBITMAP)LoadImage(NULL, fname, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE|LR_CREATEDIBSECTION);
  if(!bm_add) 
    {
      return;
    }
  GetObject(bm_add, sizeof(DIBSECTION), &ds);   // Get information
  dx = ds.dsBm.bmWidth;
  dy = ds.dsBm.bmHeight;
  if (dy < 100)
  {
    x -= dx/2;      // compute top-left to center bitmap
    y -= dy/2;
  }
  hdc_big = GetDC(NULL);               
  mdc_big = CreateCompatibleDC(hdc_big);    // Create a memory context for BITMAP
  //dmdc = CreateCompatibleDC(hdc);   // Create a memory context for DIBSECTION
  SelectObject(mdc_big, bm_big);        // select BITMAP
  //odbm = SelectBitmap(dmdc, hdibm);    // select DIBSECTION

  hdc_add = GetDC(NULL);
  mdc_add = CreateCompatibleDC(hdc_add);
  SelectObject(mdc_add, bm_add);

  BitBlt(mdc_big, x, y, dx, dy, mdc_add, 0, 0, SRCCOPY);//SRCAND);
  //SelectBitmap(dmdc, odbm);   // Release GDI resources for DIBSECITON context
  //DeleteDC(dmdc);
  //DeleteObject(hdibm);        // Delete loaded DIBSECTION

  //SelectBitmap(mdc, obm);      // Release GDI resources for BITMAP context
  DeleteDC(mdc_big);
  ReleaseDC(NULL, hdc_big);
  DeleteDC(mdc_add);
  ReleaseDC(NULL, hdc_add);
  DeleteObject(bm_add);
  //ReleaseDC(full_screen_window, hdc);
}
 

void add_line(HBITMAP hbm, int x1, int y1, int x2, int y2)
{
  HDC hdc, mdc;
  HBITMAP obm;
  HPEN newPen;

  newPen = CreatePen(PS_SOLID, 1, RGB(0, 0, 0));
  hdc = GetDC(NULL);               
  mdc = CreateCompatibleDC(hdc);
  obm = SelectObject(mdc, hbm);     
  SelectObject(mdc, newPen);
  MoveToEx(mdc, x1, y1, NULL);
  LineTo(mdc, x2, y2);
  ReleaseDC(NULL, hdc);
  SelectBitmap(mdc, obm);      // Release GDI resources for BITMAP context
  DeleteDC(mdc);  
  DeleteObject(newPen);
}

